# -*- coding: utf-8 -*-
# Generated by Django 2.2.7 on 2019-11-19 20:00

from django.db import migrations, models


class Migration(migrations.Migration):
    """Initial migration."""

    initial = True
    dependencies = []

    operations = [
        migrations.CreateModel(
            name='SomeItem',
            fields=[
                (
                    'id',
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name='ID',
                    ),
                ),
                (
                    'string_field',
                    models.CharField(max_length=50),  # noqa: WPS432
                ),
            ],
        ),
    ]
