# -*- coding: utf-8 -*-
# Generated by Django 2.2.10 on 2020-03-29 11:18

"""
This migration is named incorrectly.

We use it as a test for wrong autonames.
Please, do not rename it!
"""

from django.db import migrations


class Migration(migrations.Migration):
    """Dummy migration."""

    dependencies = [
        ('main_app', '0004_auto_20191119_2125'),
    ]

    operations = []
